<?php

namespace MetodikaTI\Http\Controllers\Back\Catalog;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Team\CreateTeamRequest;
use MetodikaTI\Http\Requests\Team\EditTeamRequest;
use MetodikaTI\Library\Pastora;
use MetodikaTI\Library\URI;
use MetodikaTI\Team;

class EquiposController extends Controller
{


    public function index()
    {

        $data = Team::get();
        return view('back.catalog.equipos.index', array('data' => $data, 'permitions' => URI::checkPermitions()));
    }


    public function create()
    {
        return view('back.catalog.equipos.create');
    }


    public function store(CreateTeamRequest $request)
    {

        //name
        //image_path

        $response = array('status' => false);


        //Primero revisamos que el nombre del equipo sea unico
        $is_team_exists = Team::where("name", mb_strtoupper($request->name));

        if ($is_team_exists->count() > 0) {
            $response['status'] = false;
            $response["message"] = "Ya existe un equipo con el nombre ingresado.";
            return response()->json($response);
        }


        DB::beginTransaction();

        $data = new Team();
        $data->name = mb_strtoupper($request->name);

        if ($data->save()) {

            DB::commit();

            //Si tiene una imagen, la cargamos
            if ($request->hasFile("image_path")) {

                $image_name = Pastora::slugify($request->name) . "_" . date("d_m_Y_H_i_s") . "." . $request->file("image_path")->extension();
                if ($request->file("image_path")->move("assets/team_logo/", $image_name)) {
                    $data->image_path = "assets/team_logo/" . $image_name;
                    $data->save();
                }

            }


            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('team_index');

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function edit($id)
    {
        $data = Team::find(base64_decode($id));

        return view('back.catalog.equipos.edit', array("data" => $data));
    }


    public function update(EditTeamRequest $request, $id)
    {

        //name

        $response = array('status' => false);

        //Primero revisamos el nombre del equipo sea unico
        $is_team_exists = Team::where("name", mb_strtoupper($request->name))->where("id", "!=", base64_decode($id));

        if ($is_team_exists->count() > 0) {
            $response['status'] = false;
            $response["message"] = "Ya existe un equipo con el nombre ingresado.";
            return response()->json($response);
        }

        DB::beginTransaction();

        $data = Team::find(base64_decode($id));
        $data->name = mb_strtoupper($request->name);

        if ($data->save()) {

            DB::commit();

            //Si tiene una imagen, la reemplazamos y cargamos
            if ($request->hasFile("image_path")) {

                $image_name = Pastora::slugify($request->name) . "_" . date("d_m_Y_H_i_s") . "." . $request->file("image_path")->extension();
                if ($request->file("image_path")->move("assets/team_logo/", $image_name)) {

                    //eliminamos la imagen anterior
                    if (file_exists($data->image_path)) {
                        unlink($data->image_path);
                    }

                    $data->image_path = "assets/team_logo/" . $image_name;
                    $data->save();
                }

            }

            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('team_index');

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function destroy($id)
    {

        $response = array(
            'status' => true,
            'message' => 'El usuario ha sido eliminado correctamente.',
            'url' => route('team_index'),
        );

        $data = Team::find(base64_decode($id));
        if ($data != null) {
            if ($data->delete()) {

                $response = [
                    'message' => 'Se ha eliminado el registro correctamente.',
                    'status' => true,
                    'url' => route('team_index')
                ];
            }
        } else {
            $response = [
                'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',
                'status' => false
            ];
        }

        return response()->json($response);
    }


}
