<?php

namespace MetodikaTI\Http\Controllers\Back\Catalog;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\Players\CreatePlayerRequest;
use MetodikaTI\Http\Requests\Back\Players\EditPlayerRequest;
use MetodikaTI\Library\URI;
use MetodikaTI\Player;
use MetodikaTI\Team;

class JugadoresController extends Controller
{


    public function index()
    {

        $data = Player::where("name", "!=", "DUMMY")->get();
        return view('back.catalog.jugadores.index', array('data' => $data, 'permitions' => URI::checkPermitions()));
    }


    public function create()
    {

        $teams = Team::pluck("name", "id");

        return view('back.catalog.jugadores.create', compact("teams"));
    }


    public function store(EditPlayerRequest $request)
    {

        //name
        //image_path

        $response = array('status' => false);

        //Primero revisamos que en conjunta los campos nombre, apellido paterno y apellido materno sean unicos
        $is_player_exists = Player::where("name", mb_strtoupper($request->name))->where("last_name", mb_strtoupper($request->last_name))->where("second_last_name", mb_strtoupper($request->second_last_name));

        if ($is_player_exists->count() > 0) {
            $response['status'] = false;
            $response["message"] = "Ya existe un jugador con el nombre y apellidos ingresados.";
            return response()->json($response);
        }


        DB::beginTransaction();

        $data = new Player();
        //$data->team_id = $request->team_id;
        $data->gender = $request->gender;
        $data->name = mb_strtoupper($request->name);
        $data->second_name = mb_strtoupper($request->second_name);
        $data->last_name = mb_strtoupper($request->last_name);
        $data->second_last_name = mb_strtoupper($request->second_last_name);
        $data->nickname = $request->nickname;
        $data->birthday = Carbon::createFromFormat("d/m/Y", $request->birthday)->format("Y-m-d");
        $data->handycap = 0;
        $data->city = $request->city;
        $data->state = $request->state;
        $data->country = $request->country;
        $data->nationality = $request->nationality;
        $data->nationality_second = $request->nationality_second;
        $data->residence_city = $request->residence_city;
        $data->cellphone = $request->cellphone;
        $data->email = $request->email;
        $data->special = $request->special;
        $data->password = md5($request->password);
        $data->nip = $request->nip;
        $data->save();


        if ($data->save()) {

            DB::commit();

            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('jugador_index');

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function edit($id)
    {
        $data = Player::find(base64_decode($id));

        $teams = Team::pluck("name", "id");

        $gender = array( array("genero" => "male" ), array("genero" => "female" ) );

        return view('back.catalog.jugadores.edit', array("data" => $data, "teams" => $teams, "gender" => $gender));
    }


    public function update(EditPlayerRequest $request, $id)
    {

        //name

        $response = array('status' => false);

        //Primero revisamos que en conjunta los campos nombre, apellido paterno y apellido materno sean unicos
        $is_player_exists = Player::where("name", mb_strtoupper($request->name))->where("last_name", mb_strtoupper($request->last_name))->where("second_last_name", mb_strtoupper($request->second_last_name))->where("id", "!=", base64_decode($id));

        if ($is_player_exists->count() > 0) {
            $response['status'] = false;
            $response["message"] = "Ya existe un jugador con el nombre y apellidos ingresados.";
            return response()->json($response);
        }

        DB::beginTransaction();

        error_log( $request->gender );

        $data = Player::find(base64_decode($id));
        //$data->team_id = $request->team_id;
        $data->gender = $request->gender;
        $data->name = mb_strtoupper($request->name);
        $data->second_name = mb_strtoupper($request->second_name);
        $data->last_name = mb_strtoupper($request->last_name);
        $data->second_last_name = mb_strtoupper($request->second_last_name);
        $data->nickname = $request->nickname;
        $data->birthday = Carbon::createFromFormat("d/m/Y", $request->birthday)->format("Y-m-d");
        $data->handycap = 0;
        $data->city = $request->city;
        $data->state = $request->state;
        $data->country = $request->country;
        $data->nationality = $request->nationality;
        $data->nationality_second = $request->nationality_second;
        $data->residence_city = $request->residence_city;
        $data->cellphone = $request->cellphone;
        $data->email = $request->email;
        $data->special = $request->special;
        $data->nip  = $request->nip;        
        
        if( $request->password ){
            $data->password = md5($request->password);
        }


        if ($data->save()) {

            DB::commit();

            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('jugador_index');

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function destroy($id)
    {

        $response = array(
            'status' => true,
            'message' => 'El usuario ha sido eliminado correctamente.',
            'url' => route('jugador_index'),
        );

        $data = Player::find(base64_decode($id));

        if ($data != null) {
            if ($data->delete()) {

                $response = [
                    'message' => 'Se ha eliminado el registro correctamente.',
                    'status' => true,
                    'url' => route('jugador_index')
                ];
            }
        } else {
            $response = [
                'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',
                'status' => false
            ];
        }

        return response()->json($response);
    }


}