<?php

namespace MetodikaTI\Http\Controllers\Back\Liga;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\CategoryRule;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\CategoryRules\CreateCategoryRulesRequest;
use MetodikaTI\Http\Requests\Back\CategoryRules\EditCategoryRulesRequest;
use MetodikaTI\League;
use MetodikaTI\Library\Pastora;
use MetodikaTI\Team;
use MetodikaTI\TeamAndCategorySpecification;
use MetodikaTI\TeamEachCategory;

class CategoryRulesController extends Controller
{

    public function create($league_id)
    {

        $league = League::find(base64_decode($league_id));
        $category_rules = CategoryRule::where("league_id", base64_decode($league_id))->first();
        $teams = Team::where("id", "!=", 39)->pluck("name", "id");

        //Si ya existe este catalogo en BD, lo redireccionamos a edit
        if ($category_rules != null) {
            return redirect(route('category_rules.edit', $league_id));
        } else {
            return view('back.league.category_rules.create', array("league" => $league, "teams" => $teams));
        }

    }


    public function store(CreateCategoryRulesRequest $request, $id)
    {

        //dd($request->all());

        $response = array('status' => false);

        DB::beginTransaction();

        $category_rules = new CategoryRule();
        $category_rules->league_id = $id;
        $category_rules->on_finish_tourment_there_are_changes_on_start_next_tourment = 0;
        $category_rules->maximum_players_by_team = $request->maximum_players_by_team;

        if ($category_rules->save()) {

            //Almacenamos los equipos que jugarán en la liga
            foreach ($request->team_name as $category_id => $teams) {

                $i = 0;
                foreach ($teams as $team_id) {
                    $team_each_category = new TeamEachCategory();
                    $team_each_category->team_category_id = $category_id;
                    $team_each_category->team_id = $team_id;
                    $team_each_category->alias_team = $i + 1;
                    $team_each_category->save();
                    $i++;
                }
            }

            DB::commit();
            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('handicap_create', base64_encode($id));

        } else {

            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }

        return response()->json($response);
    }


    public function edit($id)
    {

        $league = League::find(base64_decode($id));
        $category_rules = CategoryRule::where("league_id", base64_decode($id))->first();
        $teams = Team::where("id", "!=", 39)->pluck("name", "id");

        return view('back.league.category_rules.edit', array("league" => $league, "category_rules" => $category_rules, "teams" => $teams));
    }


    public function update(EditCategoryRulesRequest $request, $id)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        $category_rules = CategoryRule::where("league_id", $id)->first();
        $category_rules->on_finish_tourment_there_are_changes_on_start_next_tourment = 0;
        $category_rules->maximum_players_by_team = $request->maximum_players_by_team;

        if ($category_rules->save()) {

            //Almacenamos los equipos que jugarán en la liga
            foreach ($request->team_name as $team_each_categories_id => $team_ids) {
                
                foreach ($team_ids as $key => $team_id) {
                    $team_each_categories = TeamEachCategory::find($team_each_categories_id);
                    $team_each_categories->team_id = $team_id;
                    $team_each_categories->save();
                }

            }

            //Almacenamos los nuevos equipos que jugarán en la liga
            if ($request->team_name_nuevo != null) {
                foreach ($request->team_name_nuevo as $category_id => $teams) {
                    foreach ($teams as $team_id) {
                        $team_each_category = new TeamEachCategory();
                        $team_each_category->team_category_id = $category_id;
                        $team_each_category->team_id = $team_id;
                        $team_each_category->save();
                    }
                }
            }


            //Reordenamos los alias de los equipos, los alias sirven para el rol de juegos
            $team_and_category_specifications = TeamAndCategorySpecification::where("league_id", $id)->first();
            foreach ($team_and_category_specifications->get_categories as $team_categories) {
                error_log($team_categories->get_team_each_category);
                foreach ($team_categories->get_team_each_category as $key => $team) {
                    $team->alias_team = ($key + 1);
                    $team->save();
                }
            }


            DB::commit();
            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('category_rules.edit', base64_encode($id));

        } else {

            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }

        return response()->json($response);
    }


}
