<?php

namespace MetodikaTI\Http\Controllers\Back\Liga;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\DiaFestivo;
use MetodikaTI\FrecuencyCustomLeague;
use MetodikaTI\Http\Controllers\Back\DiasFestivos\DiasFestivosController;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\Rol\CreateRolRequest;
use MetodikaTI\Http\Requests\Back\Rol\EditRolRequest;
use MetodikaTI\League;
use MetodikaTI\PlayerRules;
use MetodikaTI\Rol;
use MetodikaTI\RolTournament;

class RolesController extends Controller
{

    public function create($league_id)
    {

        $league = League::find(base64_decode($league_id));
        $is_rol_generated = Rol::where("league_id", $league->id);

        //Si no se ha generado el formato de roles de equipos y jugadores, lo generamos
        if ($is_rol_generated->count() == 0) {
            return view('back.league.rol_games.create', array("league" => $league));
        } else {
            return redirect(route('rol_de_juegos.edit', $league_id));
        }

    }


    public function get_rol(Request $request, $league_id)
    {

        $league = League::find($league_id);

        $rol = $this->generate_rol($league, $request->calculate_position, $request->ronda_posiciones_type);
        //dd($rol);

        return response()->json(view('back.league.rol_games.rol', array("rol" => $rol))->render());
    }


    public function generate_rol($league, $ronda_de_posiciones, $ronda_posiciones_type)
    {

        //Generaremos el rol de juegos
        $rol_by_category = array();

        //Obtenemos las categorias
        $categories = $league->get_team_and_category_specification->get_categories;        

        
        //Por cada categoria pasaremos las fechas disponibles y asignaremos la competencia entre equipos
        foreach ($categories as $category) {

            //Generamos el array para saber que fechas se tendran los juegos
            $date_week_available = $this->list_games_dates($league, $ronda_de_posiciones, $ronda_posiciones_type, $category->id);
            
            
            //Generamos la competencia round robin
            $roundRobin = $this->roundRobin($category, $date_week_available, $ronda_de_posiciones, $league, $ronda_posiciones_type);
        
            $table_numbers = $this->tables_number($roundRobin);

            // error_log( json_encode($roundRobin) );            

            $rol = array();
            $rol["category"] = $category;
            $rol["fechas_torneo"] = $roundRobin;
            $rol["select_equipos"] = $category->get_teams_for_select();
            $rol["mesas"] = $table_numbers;
            $rol["equipos"] = $category->get_team_each_category;


            $rol_by_category[] = $rol;

        }

        //dd($rol_by_category);

        return $rol_by_category;
    }


    public function tables_number($roundRobin)
    {

        $tables = array();

        foreach ($roundRobin[0]["partidos"] as $rivality) {
            $tables[] = $rivality["Home"]["table_number"];
            $tables[] = $rivality["Away"]["table_number"];
        }

        return $tables;
    }


    public function list_games_dates($league, $ronda_de_posiciones, $ronda_posiciones_type, $category_id)
    {

        $list_of_date = array();
        $date_start_league = Carbon::createFromFormat("Y-m-d", $league->date_start_league);
        $date_end_league = Carbon::createFromFormat("Y-m-d", $league->date_end_league);
        $week_days_available = explode(",", $league->days_available);
        $week_days_number_available = array();

        //Solamente dejamos los días en numero disponibles
        foreach ($week_days_available as $key => $value) {
            if ($value == 1) {
                $week_days_number_available[] = $key;
            }
        }

        //Dependiendo de la frecuencia de la configuracion de la liga, es como iremos generando esta lista
        if ($league->frecuency == "diario") {

            //Lo haremos cada semana en los días de juego
            //do {
            //
            //    foreach ($week_days_number_available as $dia_de_la_semana_disponible) {
            //        $date_start_league->startOfWeek();
            //        $date_start_league->addDays($dia_de_la_semana_disponible);
            //        $list_of_date[] = $date_start_league->copy();
            //    }
            //
            //    $date_start_league->addWeek(1);
            //
            //} while ($date_start_league->lessThanOrEqualTo($date_end_league));

        } else if ($league->frecuency == "semana") {

            //Lo haremos "x" numero de semana en los días de juego
            do {

                foreach ($week_days_number_available as $dia_de_la_semana_disponible) {
                    $date_start_league->startOfWeek();
                    $date_start_league->addDays($dia_de_la_semana_disponible);
                    $list_of_date[] = $date_start_league->copy();
                }

                $date_start_league->addWeek(($league->frecuency_week == 0) ? 1 : $league->frecuency_week);

            } while ($date_start_league->lessThanOrEqualTo($date_end_league));

        } else if ($league->frecuency == "intervalo_fechas") {

            //$custom_dates_available = FrecuencyCustomLeague::where("league_id", $league->id)->pluck("date_available")->toArray();
            //
            //do {
            //
            //    //La fecha es uno de los dias de la semana seleccionado?
            //    if (in_array($date_start_league->dayOfWeek, $week_days_number_available)) {
            //
            //        //Si la fecha esta en los dias diponibles seleccionadas en el calendario
            //        if (in_array($date_start_league->format("Y-m-d"), $custom_dates_available)) {
            //            $list_of_date[] = $date_start_league->copy();
            //        }
            //
            //    }
            //
            //    $date_start_league->addDays(1);
            //
            //} while ($date_start_league->lessThanOrEqualTo($date_end_league));

        }


        //*** Generaremos una lista con las fechas en las que se llevara a acabo los partidos ***//
        $planeacion_fechas = array();
        $dias_festivos = $this->get_holidays($league);
        $semana_contador = 1;

        //La primera y ultima fecha siempre estaran ocupadas por la ronda inicial y ronda final
        //Ronda inicial, primera fecha
        $planeacion_fechas[] = array("fecha" => $list_of_date[0]->copy(), "type" => "ronda_inicial", "descripion" => "Ronda inicial", "semana" => $semana_contador++);

        if ($ronda_de_posiciones == 0) {

            //Configuración normal sin ronda de posiciones

            for ($i = 1; $i < count($list_of_date) - 1; $i++) {

                //Primero revisamos si la fecha no es dia festivo
                if (!array_key_exists($list_of_date[$i]->copy()->format("Y-m-d"), $dias_festivos)) {
                    //Fecha normal
                    $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "partido", "descripion" => "Partido", "semana" => $semana_contador++);
                } else {
                    //Dia festivo
                    $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "festivo", "descripion" => $dias_festivos[$list_of_date[$i]->copy()->format("Y-m-d")], "semana" => $semana_contador++);
                }

            }

        } elseif ($ronda_de_posiciones > 0 && $ronda_posiciones_type != null) {

            //Configuración con ronda de posiciones y con configuración en cada ronda de posiciones si es fisica o virtual

            $iterador_ronda_posiciones = 1;
            $iterador_ronda_posiciones_type = 0;
            $ronda_de_posiciones += 1;

            for ($i = 1; $i < count($list_of_date) - 1; $i++) {

                //Primero revisamos si la fecha no es dia festivo
                if (!array_key_exists($list_of_date[$i]->copy()->format("Y-m-d"), $dias_festivos)) {

                    if (($iterador_ronda_posiciones % $ronda_de_posiciones) == 0) {

                        if (isset($ronda_posiciones_type[$category_id][$iterador_ronda_posiciones_type])) {

                            if ($ronda_posiciones_type[$category_id][$iterador_ronda_posiciones_type] == "ronda_posiciones_virtual") {

                                //se realizo cambio primero partido

                                //Si es una ronda de posiciones virtual, se duplica la fecha
                                $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "partido", "descripion" => "Partido", "semana" => $semana_contador++);
                                $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "ronda_posiciones_virtual", "descripion" => "Ronda de posiciones", "semana" => $semana_contador);
                                $iterador_ronda_posiciones_type++;
                                $iterador_ronda_posiciones = 1;

                            } else if ($ronda_posiciones_type[$category_id][$iterador_ronda_posiciones_type] == "ronda_posiciones") {

                                //Si es ronda de posiciones fisica no afecta en el rol de juegos
                                $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "ronda_posiciones", "descripion" => "Ronda de posiciones", "semana" => $semana_contador++);
                                $iterador_ronda_posiciones_type++;
                                $iterador_ronda_posiciones = 1;

                            }

                        } else {

                            //La ronda de posiciones es nueva, la colocamos como ronda de posiciones fisica
                            $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "ronda_posiciones", "descripion" => "Ronda de posiciones", "semana" => $semana_contador++);
                            $iterador_ronda_posiciones_type++;
                            $iterador_ronda_posiciones = 1;

                        }

                    } else {
                        $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "partido", "descripion" => "Partido", "semana" => $semana_contador++);
                        $iterador_ronda_posiciones++;
                    }

                } else {
                    //Dia festivo
                    $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "festivo", "descripion" => $dias_festivos[$list_of_date[$i]->copy()->format("Y-m-d")], "semana" => $semana_contador++);
                }

            }

        } elseif ($ronda_de_posiciones > 0) {

            //Configuración con ronda de posiciones

            $iterador_ronda_posiciones = 1;
            $ronda_de_posiciones += 1;
            for ($i = 1; $i < count($list_of_date) - 1; $i++) {

                //Primero revisamos si la fecha no es dia festivo
                if (!array_key_exists($list_of_date[$i]->copy()->format("Y-m-d"), $dias_festivos)) {

                    if (($iterador_ronda_posiciones % $ronda_de_posiciones) == 0) {
                        $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "ronda_posiciones", "descripion" => "Ronda de posiciones", "semana" => $semana_contador++);
                        $iterador_ronda_posiciones = 1;
                    } else {
                        $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "partido", "descripion" => "Partido", "semana" => $semana_contador++);
                        $iterador_ronda_posiciones++;
                    }

                } else {
                    //Dia festivo
                    $planeacion_fechas[] = array("fecha" => $list_of_date[$i]->copy(), "type" => "festivo", "descripion" => $dias_festivos[$list_of_date[$i]->copy()->format("Y-m-d")], "semana" => $semana_contador++);
                }

            }

        }

        //Ronda final, ultima fecha
        $planeacion_fechas[] = array("fecha" => $list_of_date[count($list_of_date) - 1]->copy(), "type" => "ronda_final", "descripion" => "Ronda final", "semana" => $semana_contador++);

        //dd($planeacion_fechas);

        return $planeacion_fechas;
    }


    public function roundRobin($category, $fechas_planeacion, $ronda_posiciones, $league, $ronda_posiciones_type)
    {

        $teams = $category->get_team_each_category->toArray();

        $mesas = array();


        //*****************Generamos la distribucion round robin
        //si es impart, agregamos el equipo Dummy

        if (count($teams) % 2 != 0) {

            $dummy = array(
                "id" => 0,
                "team_category_id" => 0,
                "team_name" => "DUMMY",
                "created_at" => "",
                "updated_at" => "",
                "table_number" => "",
                "team_id" => 0
            );

            array_push($teams, $dummy);
        }


        //Para desarrollo, vamos a asignarle un numero entero a cada equipo
        for ($i = 0; $i < count($teams); $i++) {
            $teams[$i]["ITERATOR"] = $i + 1;
        }


        //Reordenamos el array para que comience en el index 1
        $teams = array_combine(range(1, count($teams)), array_values($teams));


        //Hacemos la iteracion horizontal para ir generando los partidos
        $last_team_in_row = 1;
        $round = array();
        for ($fecha = 1; $fecha < count($teams); $fecha++) {

            //Mesa en la que iniciaran los equipos
            $start_table_number = 1;
            if ($category->is_table_asigned == 1) {
                $start_table_number = $category->table_number_start;
            }

            $horizontal_versus = array();

            //Revisamos si la fecha es par o impar
            if ($fecha % 2 == 0) {

                $horizontal_versus[1] = $teams[count($teams)];
                $horizontal_versus[2] = $teams[$last_team_in_row];

                $consecutive_iterator = $last_team_in_row + 1;

                //Iteracion de equipos hacia la derecha -->
                for ($partido = 3; $partido < count($teams); $partido += 2, $consecutive_iterator++) {

                    if ($consecutive_iterator >= count($teams)) {
                        $consecutive_iterator = 1;
                    }

                    $horizontal_versus[$partido] = $teams[$consecutive_iterator];

                }

                $last_team_in_row = $consecutive_iterator;

                //Iteracion de equipos hacia la izqueirda <--
                for ($partido = count($teams); $partido > 2; $partido -= 2, $consecutive_iterator++) {  //Mayor a 2 ya que la casilla 1 y 2 las llenas al principio

                    if ($consecutive_iterator >= count($teams)) {
                        $consecutive_iterator = 1;
                    }

                    $horizontal_versus[$partido] = $teams[$consecutive_iterator];

                }

                //Reordenamos por los indices el array
                ksort($horizontal_versus);

                //Iteramos los partidos para hacer la separacion de 2 en 2 para Home y Away
                $home_away_array = array();
                for ($i = 1; $i < count($horizontal_versus); $i += 2) {
                    $home_away_array[] = array(
                        "Home" => array(
                            "table_number" => $start_table_number++,
                            "team_id" => $horizontal_versus[$i]["team_id"],
                            "ITERATOR" => $horizontal_versus[$i]["ITERATOR"]
                        ),
                        "Away" => array(
                            "table_number" => $start_table_number++,
                            "team_id" => $horizontal_versus[$i + 1]["team_id"],
                            "ITERATOR" => $horizontal_versus[$i + 1]["ITERATOR"]
                        )
                    );
                }

                $round[$fecha] = $home_away_array;

            } else {

                $horizontal_versus[1] = $teams[$last_team_in_row];
                $horizontal_versus[2] = $teams[count($teams)];

                $consecutive_iterator = $last_team_in_row + 1;

                //Iteracion de equipos hacia la derecha -->
                for ($partido = 3; $partido < count($teams); $partido += 2, $consecutive_iterator++) {

                    if ($consecutive_iterator >= count($teams)) {
                        $consecutive_iterator = 1;
                    }

                    $horizontal_versus[$partido] = $teams[$consecutive_iterator];

                }

                $last_team_in_row = $consecutive_iterator;

                //Iteracion de equipos hacia la izqueirda <--
                for ($partido = count($teams); $partido > 2; $partido -= 2, $consecutive_iterator++) {  //Mayor a 2 ya que la casilla 1 y 2 las llenas al principio

                    if ($consecutive_iterator >= count($teams)) {
                        $consecutive_iterator = 1;
                    }

                    $horizontal_versus[$partido] = $teams[$consecutive_iterator];

                }

                //Reordenamos por los indices el array
                ksort($horizontal_versus);

                //Iteramos los partidos para hacer la separacion de 2 en 2 para Home y Away
                $home_away_array = array();
                for ($i = 1; $i < count($horizontal_versus); $i += 2) {
                    $home_away_array[] = array(
                        "Home" => array(
                            "table_number" => $start_table_number++,
                            "team_id" => $horizontal_versus[$i]["team_id"],
                            "ITERATOR" => $horizontal_versus[$i]["ITERATOR"]
                        ),
                        "Away" => array(
                            "table_number" => $start_table_number++,
                            "team_id" => $horizontal_versus[$i + 1]["team_id"],
                            "ITERATOR" => $horizontal_versus[$i + 1]["ITERATOR"]
                        )
                    );
                }

                $round[$fecha] = $home_away_array;

            }


        }

        //Para la ronda inicial vamos a colocar la rivalidad de los equipos como se dieron de alta en la seccion de "Reglas de categorias"
        $start_table_number = 1;
        if ($category->is_table_asigned == 1) {
            $start_table_number = $category->table_number_start;
        }

        $round_ronda_inicial = array();
        for ($i = 1; $i <= count($teams); $i += 2) {
            $home_away_array = array(
                "Home" => array(
                    "table_number" => $start_table_number++,
                    "team_id" => $teams[$i]["team_id"],
                    "ITERATOR" => $teams[$i]["ITERATOR"]
                ),
                "Away" => array(
                    "table_number" => $start_table_number++,
                    "team_id" => $teams[$i + 1]["team_id"],
                    "ITERATOR" => $teams[$i + 1]["ITERATOR"]
                )
            );            

            $round_ronda_inicial[] = $home_away_array;

        }


        //*****************Asignamos las fechas a cada partido y la ronda de posiciones  $round
        //Si va a ver ronda de posiciones, incrementamos +1 su valor ya que la fecha de ronda de posiciones es una semana siguiente.
        //$ronda_posiciones = ($ronda_posiciones != 0) ? ($ronda_posiciones + 1) : 0;

        //$lista_partidos = array_fill(0, count($fechas_partidos), NULL);

        $colspan_size = count($round) + (count($round) / 2) + 1;

        //Siempre la primer fecha será ronda inicial y la ultima ronda final
        //Primer fecha
        //$lista_partidos[0] = array(
        //    "fecha" => $fechas_partidos[0]->copy(),
        //    "type" => "ronda_inicial",
        //    "partidos" => $this->getRoundRobinByRondaDePosiciones($category),
        //    "colspan" => $colspan_size,
        //);

        //Ultima fecha
        //$lista_partidos[count($lista_partidos) - 1] = array(
        //    "fecha" => $fechas_partidos[count($fechas_partidos) - 1]->copy(),
        //    "type" => "ronda_final",
        //    "partidos" => $this->getRoundRobinByRondaDePosiciones($category),
        //    "colspan" => $colspan_size,
        //);

        $iterador_equipos = 1;

        //Obtenemos los días festivos
        //$dias_festivos = $this->get_holidays($league);
        //$iterador_ronda_posiciones = 1;

        // 1 = por que la primer fecha ya la asignamos como ronda inicial.
        // count($fechas_partidos) - 1 =  por que la ultima fecha ya la asignamos como ronda final.
        //dd($fechas_partidos);

        foreach ($fechas_planeacion as $key => $fecha) {

            if (count($round) == $iterador_equipos) {
                $iterador_equipos = 1;
            }

            if ($fecha["type"] == "ronda_inicial") {

                $fechas_planeacion[$key]["colspan"] = $colspan_size;
                $fechas_planeacion[$key]["partidos"] = $round_ronda_inicial; //En la ronda inicial colocamos el calculo customizado para esta ronda

            } else if ($fecha["type"] == "partido") {

                $fechas_planeacion[$key]["colspan"] = $colspan_size;
                $fechas_planeacion[$key]["partidos"] = $round[$iterador_equipos];
                $iterador_equipos++;

            } else if ($fecha["type"] == "festivo") {

                $fechas_planeacion[$key]["colspan"] = $colspan_size;

            } else if ($fecha["type"] == "ronda_posiciones") {

                $fechas_planeacion[$key]["colspan"] = $colspan_size;
                $fechas_planeacion[$key]["partidos"] = $round[1]; //En ronda de posiciones no sabemos que partidos habran pero colocamos la lista de partidos para tener un "template" de cuantos equipos jugaran.

            } else if ($fecha["type"] == "ronda_posiciones_virtual") {

                $fechas_planeacion[$key]["colspan"] = $colspan_size;
                $fechas_planeacion[$key]["partidos"] = $round[1]; //En ronda de posiciones virtual no sabemos que partidos habran pero colocamos la lista de partidos para tener un "template" de cuantos equipos jugaran.

            } else if ($fecha["type"] == "ronda_final") {

                $fechas_planeacion[$key]["colspan"] = $colspan_size;
                $fechas_planeacion[$key]["partidos"] = $round[1]; //En ronda final no sabemos que partidos habran pero colocamos la lista de partidos para tener un "template" de cuantos equipos jugaran.

            }

            /*
            //Primero revisamos si la fecha no es dia festivo
            if (!array_key_exists($fechas_partidos[$i]->copy()->format("Y-m-d"), $dias_festivos)) {

                //Si contiene ronda de posiciones y es la fecha (cada numero de semanas que contenga $ronda_posiciones) en la que tenemos que poner como ronda de posiciones, asignamos la fecha como ronda de posiciones
                if ($ronda_posiciones != 0 && ($iterador_ronda_posiciones % $ronda_posiciones) == 0) {

                    //En caso de que $ronda_posiciones_type sea diferente de null, verificamos si en la configuración se coloco que sea ronda_posiciones op ronda_posiciones_virtual
                    $configuration_type = $this->getConfigurationType($category, $i, $ronda_posiciones_type);

                    //Se van a ir repitiendo todas las rondas de partidas para completar el calendario, en caso de que lleguemos hasta el tope de series de partidos, reiniciamos la variable para volver a comenzar
                    //Ronda de posiciones fisica
                    if (count($round) == $iterador_equipos) {
                        $iterador_equipos = 1;
                    }

                    $lista_partidos[$i] = array(
                        "fecha" => $fechas_partidos[$i]->copy(),
                        "type" => "ronda_posiciones",
                        //"partidos" => $round[$iterador_equipos],
                        "colspan" => $colspan_size,
                    );

                    $iterador_equipos++;
                    $iterador_ronda_posiciones = 1;

                } else {

                    //Se van a ir repitiendo todas las rondas de partidas para completar el calendario, en caso de que lleguemos hasta el tope de series de partidos, reiniciamos la variable para volver a comenzar
                    if (count($round) == $iterador_equipos) {
                        $iterador_equipos = 1;
                    }

                    $lista_partidos[$i] = array(
                        "fecha" => $fechas_partidos[$i]->copy(),
                        "type" => "partido",
                        "partidos" => $round[$iterador_equipos],
                        "colspan" => $colspan_size,
                    );

                    $iterador_equipos++;
                    $iterador_ronda_posiciones++;

                }

            } else {

                //Es día festivo
                $lista_partidos[$i] = array(
                    "fecha" => $fechas_partidos[$i]->copy(),
                    "type" => "dia_festivo",
                    "descripcion" => $dias_festivos[$fechas_partidos[$i]->copy()->format("Y-m-d")],
                    "colspan" => $colspan_size,
                );

            }
            */

        }


        //Hay una regla de si existen rondas de posiciones por ejemplo cada 4 semanas y si el total de semanas no es multiplo de 4 y al final sobraron 3 semanas, hay que poner
        //la ronda de posiciones en la ultima semana. Ver archivo explicacion_ronda_posiciones.png
        //$ronda_posiciones != 0 : Si va a ver ronda de posiciones
        //(($iterador_ronda_posiciones - 1) != ($ronda_posiciones - 1)) Si en la ultima ronda antes de poner la ronda de posiciones no fue multiplo (Ver archivo explicacion_ronda_posiciones.png)
        //if ($ronda_posiciones != 0 && (($iterador_ronda_posiciones - 1) != ($ronda_posiciones - 1))) {
        //    $lista_partidos[count($lista_partidos) - 2]["type"] = "ronda_posiciones";
        //}


        return $fechas_planeacion;
    }


    public function getConfigurationType($category, $key_fechas_torneo, $ronda_posiciones_type)
    {

        $category_id = $category->id;
        $type_selected = null;

        foreach ($ronda_posiciones_type as $item) {
            if ($item["category_id"] == $category_id && $item["key_fechas_torneo"] == $key_fechas_torneo) {
                $type_selected = $item["type_selected"];
            }
        }

        return $type_selected;
    }


    public function getRoundRobinByRondaDePosiciones($category)
    {

        //Aqui tenemos que iterar la ronda de posiciones acorde al lugar que tienen en la tabla
        $start_table_number = 1;
        $teams = $category->get_team_each_category->toArray();

        //si es impart, agregamos el equipo Dummy
        if (count($teams) % 2 != 0) {

            $dummy = array(
                "id" => 0,
                "team_category_id" => 0,
                "team_name" => "DUMMY",
                "created_at" => "",
                "updated_at" => "",
                "table_number" => "",
                "team_id" => 0
            );

            array_push($teams, $dummy);
        }

        if ($category->is_table_asigned == 1) {
            $start_table_number = $category->table_number_start;
        }

        //Hacemos la distribucion de competencia
        $round = array();
        $iterator = 0;
        for ($i = 0; $i < count($teams); $i += 2) {

            $teams[$i]["table_number"] = $start_table_number++;
            $teams[$i + 1]["table_number"] = $start_table_number++;

            $round[$iterator]["Home"] = $teams[$i];
            $round[$iterator]["Away"] = $teams[$i + 1];

            $iterator++;
        }

        return $round;
    }


    public function get_holidays($league)
    {

        $dias_festivos = array();

        //Días festivos del catalogo
        $dias_festivos_catalogo = DiaFestivo::get();
        foreach ($dias_festivos_catalogo as $key => $value) {
            $dias_festivos[$value->date] = $value->name;
        }

        //Días festivos de la liga
        $días_festivos_liga = explode(",", $league->holidays_date);
        foreach ($días_festivos_liga as $key => $value) {
            if ($value != "") {
                $dias_festivos[Carbon::createFromFormat("d/m/Y", $value)->copy()->format("Y-m-d")] = "Fecha omitida en la configuración de la liga";
            }
        }

        return $dias_festivos;
    }


    public function store(CreateRolRequest $request, $id)
    {

        //dd($request->all());

        $response = array('status' => false);
        $league = League::find($id);

        DB::beginTransaction();

        $rol = new Rol();
        $rol->league_id = $league->id;
        $rol->calculate_position = $request->calculate_position;
        $rol->can_team_ausent = ($request->can_team_ausent == "si") ? 1 : 0;
        $rol->how_many_ausent = ($request->can_team_ausent == "si") ? $request->how_many_ausent : 0;
        $rol->save();

        // error_log( $request->date_tourment );

        //Ingresamos todo_ el rol de juegos
        foreach ($request->date_tourment as $category_id => $tourment_by_category) {

            foreach ($tourment_by_category as $iterator_week => $tournament_by_week) {

                //dd("category_id:" . $category_id, "iterator_week" . $iterator_week, $tournament_by_week);

                //Todos los tipos de registro tienen la misma estructura salvo cuando sea festivo
                if ($tournament_by_week["row_type"] != "festivo") {

                    foreach ($tournament_by_week["rivalidad_partido"] as $game) {

                        $rol_tournaments = new RolTournament();
                        $rol_tournaments->rol_id = $rol->id;
                        $rol_tournaments->category_id = $category_id;
                        $rol_tournaments->iterator_week = $iterator_week;
                        $rol_tournaments->date = Carbon::createFromFormat("d/m/Y", $tournament_by_week["fecha_torneo"])->format("Y-m-d");
                        $rol_tournaments->type = $tournament_by_week["row_type"];

                        $iterador_game = 1;
                        foreach ($game as $table_number => $team_id) {

                            if ($iterador_game == 1) {
                                $rol_tournaments->first_table_number = $table_number;
                                $rol_tournaments->first_team_id = $team_id["team_id"];
                            } elseif ($iterador_game == 2) {
                                $rol_tournaments->second_table_number = $table_number;
                                $rol_tournaments->second_team_id = $team_id["team_id"];
                            }

                            $iterador_game++;

                        }

                        $rol_tournaments->save();

                    }

                } else {

                    $rol_tournaments = new RolTournament();
                    $rol_tournaments->rol_id = $rol->id;
                    $rol_tournaments->category_id = $category_id;
                    $rol_tournaments->iterator_week = $iterator_week;
                    $rol_tournaments->date = Carbon::createFromFormat("d/m/Y", $tournament_by_week["fecha_torneo"])->format("Y-m-d");
                    $rol_tournaments->type = $tournament_by_week["row_type"];
                    $rol_tournaments->save();

                }

            }

        }

        DB::commit();

        $response['status'] = true;
        $response["message"] = "Los datos se han almacenado correctamente.";
        $response['url'] = route('league_index');

        return response()->json($response);
    }


    public function edit($id)
    {

        $league = League::find(base64_decode($id));
        $rol_data = Rol::where("league_id", $league->id)->first();
        // dd($rol_data);
        $rol = array();

        //Generamos el array como cuando genera el rol de juegos
        //Iteramos por cada categoria
        foreach ($league->get_team_and_category_specification->get_categories as $category) {

            // error_log( print_r($category, true) );

            $rol_content = array("category" => array(), "fechas_torneo" => array(), "select_equipos" => array(), "mesas" => array(), "equipos" => array());

            //************************************************************
            //************************************ CATEGORY
            //************************************************************
            $rol_content["category"] = $category;


            //************************************************************
            //************************************ SELECT EQUIPOS
            //************************************************************
            $rol_content["select_equipos"] = $category->get_teams_for_select();


            //************************************************************
            //************************************ MESAS
            //************************************************************
            //Generamos el array para las mesas dependiendo de la cantidad de equipos por categoria
            $mesas = array();
            $table_number_start = 1;

            if ($category->is_table_asigned == 1) {
                $table_number_start = $category->table_number_start;
            }

            for ($i = 0; $i < $category->get_team_each_category->count(); $i++) {
                $mesas[] = $table_number_start;
                $table_number_start++;
            }

            $rol_content["mesas"] = $mesas;


            //************************************************************
            //************************************ EQUIPOS
            //************************************************************
            $rol_content["equipos"] = $category->get_team_each_category;


            //************************************************************
            //************************************ FECHAS TORNEO
            //************************************************************
            $fechas_torneo = array();
            $total_groups_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->groupBy("iterator_week")->get()->count();

            error_log("total_groups_iterator_week =>". $rol_data->id. "----". $category->id);

            $iterador_semana = 1;
            for ($i = 0; $i < $total_groups_iterator_week; $i++) {

                $contenido_fecha_torneo = array("fecha" => "", "type" => "", "partidos" => "");
                $games_in_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->where("iterator_week", $i)->orderBy("id")->get();
                // error_log(json_encode($games_in_iterator_week));
                $first_game_in_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->where("iterator_week", $i)->orderBy("id")->first(); //Este nos servidora para obtener la fecha y el tipo

                $contenido_fecha_torneo["fecha"] = Carbon::createFromFormat("Y-m-d", $first_game_in_iterator_week->date);
                $contenido_fecha_torneo["type"] = $first_game_in_iterator_week->type;
                $contenido_fecha_torneo["colspan"] = count($category->get_team_each_category) + ((count($category->get_team_each_category) / 2) - 1);

                //Obtenemos la descripcion del tipo de fila
                if ($first_game_in_iterator_week->type == "festivo") {
                    $contenido_fecha_torneo["descripion"] = $this->get_festivo_description($first_game_in_iterator_week->date);
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "partido") {
                    $contenido_fecha_torneo["descripion"] = "Partido";
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "ronda_final") {
                    $contenido_fecha_torneo["descripion"] = "Ronda final";
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "ronda_inicial") {
                    $contenido_fecha_torneo["descripion"] = "Ronda inicial";
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "ronda_posiciones") {
                    $contenido_fecha_torneo["descripion"] = "Ronda de posiciones";

                    if (($i == ($total_groups_iterator_week - 2))) {
                        $contenido_fecha_torneo["semana"] = $iterador_semana -= 1;
                        $iterador_semana++;
                    } else {
                        $contenido_fecha_torneo["semana"] = $iterador_semana++;
                    }

                } elseif ($first_game_in_iterator_week->type == "ronda_posiciones_virtual") {

                    $contenido_fecha_torneo["descripion"] = "Ronda de posiciones virtual";

                    //Para el caso cuando se agrega la ronda de posiciones al final, revisaremos si estamos iterando dicho registro para evaluar el numero de semana
                    if (($i == ($total_groups_iterator_week - 2))) {
                        $contenido_fecha_torneo["semana"] = $iterador_semana -= 1;
                        $iterador_semana++;
                    } else {
                        $contenido_fecha_torneo["semana"] = $iterador_semana;
                    }
                }

                $games_in_week = array();
                foreach ($games_in_iterator_week as $games) {

                    $games_in_week[] =
                        array(
                            "Home" =>
                                array("id" => $games->id, "team_category_id" => $games->category_id, "team_id" => $games->first_team_id, "table_number" => $games->first_table_number),
                            "Away" =>
                                array("id" => $games->id, "team_category_id" => $games->category_id, "team_id" => $games->second_team_id, "table_number" => $games->second_table_number)
                        );

                }

                $contenido_fecha_torneo["partidos"] = $games_in_week;

                $fechas_torneo[] = $contenido_fecha_torneo;

            }

            $rol_content["fechas_torneo"] = $fechas_torneo;

            $rol[] = $rol_content;
        }

        // dd($rol);

        $rol_view = view('back.league.rol_games.rol', array("rol" => $rol))->render();
        
        return view('back.league.rol_games.edit', array("league" => $league, "rol_data" => $rol_data, "rol" => $rol, "rol_view" => $rol_view));
    }


    public function get_festivo_description($date)
    {

        //Si es festivo, buscamos que fecha es
        $festivo = DiaFestivo::where("date", $date)->first();
        $description = "";

        if ($festivo != null) {
            $description = $festivo->name;
        } else {
            $description = "Fecha omitida en la configuración de la liga";
        }

        return $description;
    }


    public function update(EditRolRequest $request, $id)
    {   

        error_log("ID =========> ".$id);

        $response = array('status' => false);
        $league = League::find($id);

        DB::beginTransaction();

        $rol = Rol::where("league_id", $league->id)->first();
        $rol->calculate_position = $request->calculate_position;
        $rol->can_team_ausent = ($request->can_team_ausent == "si") ? 1 : 0;
        $rol->how_many_ausent = ($request->can_team_ausent == "si") ? $request->how_many_ausent : 0;
        $rol->save();

        //se se añadio 1

        //Eliminamos todo_ el rol_tournaments para volverlo a agregar en caso de que halla sufrido cambios
        RolTournament::where("rol_id", $rol->id)->delete();

        error_log( "date_tourment ==>". json_encode($request->date_tourment) );

        //Ingresamos todo_ el rol de juegos
        foreach ($request->date_tourment as $category_id => $tourment_by_category) {

            foreach ($tourment_by_category as $iterator_week => $tournament_by_week) {

                //dd("category_id:" . $category_id, "iterator_week" . $iterator_week, $tournament_by_week);

                //Todos los tipos de registro tienen la misma estructura salvo cuando sea festivo
                if ($tournament_by_week["row_type"] != "festivo") {

                    foreach ($tournament_by_week["rivalidad_partido"] as $game) {

                        $rol_tournaments = new RolTournament();
                        $rol_tournaments->rol_id = $rol->id;
                        $rol_tournaments->category_id = $category_id;
                        $rol_tournaments->iterator_week = $iterator_week;
                        $rol_tournaments->date = Carbon::createFromFormat("d/m/Y", $tournament_by_week["fecha_torneo"])->format("Y-m-d");
                        $rol_tournaments->type = $tournament_by_week["row_type"];

                        $iterador_game = 1;
                        foreach ($game as $table_number => $team_id) {

                            if ($iterador_game == 1) {
                                $rol_tournaments->first_table_number = $table_number;
                                $rol_tournaments->first_team_id = $team_id["team_id"];
                            } elseif ($iterador_game == 2) {
                                $rol_tournaments->second_table_number = $table_number;
                                $rol_tournaments->second_team_id = $team_id["team_id"];
                            }

                            $iterador_game++;

                        }

                        $rol_tournaments->save();

                    }

                } else {

                    $rol_tournaments = new RolTournament();
                    $rol_tournaments->rol_id = $rol->id;
                    $rol_tournaments->category_id = $category_id;
                    $rol_tournaments->iterator_week = $iterator_week;
                    $rol_tournaments->date = Carbon::createFromFormat("d/m/Y", $tournament_by_week["fecha_torneo"])->format("Y-m-d");
                    $rol_tournaments->type = $tournament_by_week["row_type"];
                    $rol_tournaments->save();

                }

            }

        }

        DB::commit();

        $response['status'] = true;
        $response["message"] = "Los datos se han almacenado correctamente.";
        $response['url'] = route('rol_de_juegos.edit', base64_encode($id));


        return response()->json($response);
    }

    public function edit_rol_team( Request $request ){

        $response["status"] = true;
        $response["message"] = "El rol de juegos se actualizó correctamente.";
        
        $rol = Rol::where("league_id", $request->league_id)->first();
        
        if( $request->data ){
            
            foreach ($request->data as $key => $value) {                
                // [home_away] DETERMINA SI ES EL PRIMER EQUIPO(1) O EL SEGUNDO EQUIPO (2)                                
                $update = RolTournament::where("rol_id",$rol->id)->where("id", $value["id_semana"])->update( $this->home_away( $value["home_away"], $value["new_equipo_id"] ) );                
            }            

        }

        return response()->json($response);   
    }

    private function home_away( $valor, $id ){
        if( $valor == 1 ){
            return ['first_team_id' => $id];
        }else{
            return ['second_team_id' => $id];
        }
    }

}
