<?php

namespace MetodikaTI;

use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;

class TeamCategory extends Model
{

    protected $appends = ["table_number_initial"];

    public function get_team_each_category()
    {
        return $this->hasMany(TeamEachCategory::class, "team_category_id", "id");
    }


    public function get_teams_for_select()
    {

        $teams_in_select = array();
        $teams_in_select = $this->hasMany(TeamEachCategory::class, "team_category_id", "id")->pluck("alias_team", "team_id");

        if (count($teams_in_select) % 2 != 0) {
            $teams_in_select[0] = 20;
        }

        return $teams_in_select;
    }

    public function get_handicap()
    {
        return $this->hasOne(HandicapEachCategory::class, "team_category_id", "id");
    }

    public function get_awards_are_by_hierarchy()
    {
        return $this->hasOne(PremiacionEachCategory::class, "team_category_id", "id");
    }

    public function get_how_many_teams_get_award()
    {
        return $this->hasOne(PremiacionEachCategory::class, "team_category_id", "id");
    }

    public function get_rule_for_individual_award()
    {
        return $this->hasOne(PremiacionEachCategory::class, "team_category_id", "id");
    }

    public function get_award_game_high_by_team()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "game_high_by_team")->orderBy("place", "asc");
    }

    public function get_award_game_high_by_team_with_handicap()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "game_high_by_team_with_handicap")->orderBy("place", "asc");
    }

    public function get_award_serie_high_by_team()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "serie_high_by_team")->orderBy("place", "asc");
    }

    public function get_award_serie_high_by_team_with_handicap()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "serie_high_by_team_with_handicap")->orderBy("place", "asc");
    }

    public function get_award_by_team()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "award_by_team")->orderBy("place", "asc");
    }

    public function get_award_game_high_individual_male()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "game_high_individual_male")->orderBy("place", "asc");
    }

    public function get_award_game_high_individual_male_with_handicap()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "game_high_individual_male_with_handicap")->orderBy("place", "asc");
    }

    public function get_award_serie_high_individual_male()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "serie_high_individual_male")->orderBy("place", "asc");
    }

    public function get_award_serie_high_individual_male_with_handicap()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "serie_high_individual_male_with_handicap")->orderBy("place", "asc");
    }

    public function get_award_average_individual_male()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "average_individual_male")->orderBy("place", "asc");
    }

    public function get_award_game_high_individual_female()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "game_high_individual_female")->orderBy("place", "asc");
    }

    public function get_award_game_high_individual_female_with_handicap()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "game_high_individual_female_with_handicap")->orderBy("place", "asc");
    }

    public function get_award_serie_high_individual_female()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "serie_high_individual_female")->orderBy("place", "asc");
    }

    public function get_award_serie_high_individual_female_with_handicap()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "serie_high_individual_female_with_handicap")->orderBy("place", "asc");
    }

    public function get_award_average_individual_female()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "average_individual_female")->orderBy("place", "asc");
    }

    public function get_award_perfect_game()
    {
        return $this->hasMany(PremiacionLugares::class, "category_id", "id")->where("type", "perfect_game")->orderBy("place", "asc");
    }

    public function getTableNumberInitialAttribute()
    {

        if ($this->is_table_asigned == 1) {
            return $this->table_number_start;
        } else {
            return 1;
        }

    }

    public function get_team_and_category_specifications()
    {
        return $this->hasMany(TeamAndCategorySpecification::class, "id", "team_and_category_specifications_id");
    }

}
